/*  CXL - Copyright (c) 1987-1989 by Mike Smedley - All Rights Reserved  */
/*  WPUTSW.C   - puts a string in active window utilizing word wrap  */

#include "cxlwin.h"

#define MAXWORDSIZE 255

static char wordbuf[MAXWORDSIZE+1];
static int  bufptr=0;

static void putword(void);

int wputsw(char *str)
{
    char *p;
    int lastrow;

    /* check for active window */
    if(!_winfo.total) return(_winfo.errno=W_NOACTIVE);

    /* do while more characters */
    for(p=str;*p;p++) {

        /* if current position + buffer pointer > right border */
        /* then write word buffer then advance to next line    */
        if( (_winfo.active->column+bufptr) >
           (_winfo.active->ecol-_winfo.active->border) ) {

            /* if word can't fit in window, break it up */
            if( (_winfo.active->scol+_winfo.active->border+bufptr) >
            (_winfo.active->ecol-_winfo.active->border) ) {
                lastrow=_winfo.active->row;
                putword();

                /* see if new line was already done by wputs(). */
                /* if not, then manually write a new line.      */
                if(_winfo.active->row<=lastrow) wputc('\n');
                if(*p!=' ') wordbuf[bufptr++]=*p;
            }

            /* otherwise wrap word around */
            else {
                wputc('\n');
                putword();
                wordbuf[bufptr++]=*p;
            }
        }
        else {

            /* if character is a space */
            if(*p==' ') {

                /* if not at first window column, add the    */
                /* character to buffer and write word buffer */
                if( (_winfo.active->column+bufptr) !=
                  (_winfo.active->srow+_winfo.active->border) ) {
                    wordbuf[bufptr++]=' ';
                    putword();
                }
            }
            else {

                /* write current character to word buffer */
                wordbuf[bufptr++]=*p;
            }
        }
    }

    /* flush word buffer */
    putword();

    /* return normally */
    return(_winfo.errno=W_NOERROR);
}

/*---------------------------------------------------------------------------*/

static void putword(void)
{
    if(bufptr) {
        wordbuf[bufptr]='\0';
        wputs(wordbuf);
        bufptr=0;
    }
}

